### Process flood data and precipitation data to annual times series selecting the season and forming Rx5d and Rx20d 
library(stringr)
library(MASS)

movingAverage <- function(x, n=1, centered=FALSE) {
    if (centered) {before <- floor  ((n-1)/2)
    after  <- ceiling((n-1)/2)
    } else {before <- n-1
    after  <- 0}
    s     <- rep(0, length(x))
    count <- rep(0, length(x))
    new <- x
    count <- count + !is.na(new)
    new[is.na(new)] <- 0
    s <- s + new
    i <- 1
    while (i <= before) {
        new   <- c(rep(NA, i), x[1:(length(x)-i)])
        count <- count + !is.na(new)
        new[is.na(new)] <- 0
        s <- s + new
        i <- i+1}
    i <- 1
    while (i <= after) {
        new   <- c(x[(i+1):length(x)], rep(NA, i))
        count <- count + !is.na(new)
        new[is.na(new)] <- 0
        s <- s + new
        i <- i+1}
    s/count}

### read in data
lst <- read.table("filelist.txt",header=F)
day <- read.table("day.txt")

### initialize arrays, write time column
ap <- array(NA,dim=c(length(1780:2021),dim(lst)[1]+1))
apg <- array(NA,dim=c(length(1780:2021),dim(lst)[1]+1))
af <- array(NA,dim=c(length(1780:2021),dim(lst)[1]+1))
afg <- array(NA,dim=c(length(1780:2021),dim(lst)[1]+1))
ap[,1] <- 1780:2021
apg[,1] <- 1780:2021
af[,1] <- 1780:2021
afg[,1] <- 1780:2021

### main loop
for (i in 1:length(lst[,1])){
 ## process precipitation
 x <- read.table(lst[i,2],header=F)
 if (lst[i,3]=="MJJASO"){selseas <- x[,2]>4&x[,2]<11}else{selseas <- !(x[,2]>4&x[,2]<11)}
 ner <- x[,4]<0 ## remove negative precip, NA 
 ner[is.na(ner)]<- T
 x[ner,4] <- NA
 x[!selseas,4] <- NA
 xa <- rep(NA,length(day[,1]))
 xa[match(10000*x[,1]+100*x[,2]+x[,3],10000*day[,1]+100*day[,2]+day[,3])]<-x[,4]
 x <- as.matrix(cbind(day,xa))
 x5d <- as.numeric(movingAverage(x[,4],5,centered=T))
 x20d <- as.numeric(movingAverage(x[,4],20,centered=T))
 xc <- aggregate(!is.na(x[,4]),list(x[,1]),sum,na.rm=T) ## count available observations
 x5m <- aggregate(x5d,list(x[,1]),max,na.rm=T)
 x5m[xc[,2]<165,2] <- NA ## kick out years with fewer than 165 days w. measurements per half-year
 x20m <- aggregate(x20d,list(x[,1]),max,na.rm=T)
 x20m[xc[,2]<165,2] <- NA ## kick out years with fewer than 165 days w. measurements per half-year
 x5mm <- aggregate(x5d,list(100*x[,1]+x[,2]),max,na.rm=T)
 x5mm[x5mm[,2]==-Inf,2] <- NA
 x20mm <- aggregate(x20d,list(100*x[,1]+x[,2]),max,na.rm=T)
 x20mm[x20mm[,2]==-Inf,2] <- NA
 ap[,i+1] <- x5m[,2]
 xout <- cbind(1780:2021,ap[,i+1])
 xmf <- ap[c(1780:2021)>1799&!is.na(ap[,i+1]),i+1] ## cut from 1800
 g <- fitdistr(xmf, "gamma")
 xout[!is.na(ap[,i+1]),2] <- qnorm(pgamma(ap[!is.na(ap[,i+1]),i+1],g$estimate[1],g$estimate[2]))
 write.table(xout,str_replace(lst[i,2], "dailyprecip.txt","Rx5d_gamma_season.txt"),sep=" ",row.names=F,col.names=F)
 apg[,i+1] <- xout[,2]
 h5m <- hist(x[!is.na(x5d),2][match(1000*x5m[!is.na(x5m[,2]),1]+x5m[!is.na(x5m[,2]),2],1000*x[!is.na(x5d),1]+x5d[!is.na(x5d)])],plot=F,breaks=c(0:12)+0.5)$counts
 h20m <- hist(x[!is.na(x20d),2][match(1000*x20m[!is.na(x20m[,2]),1]+x20m[!is.na(x20m[,2]),2],1000*x[!is.na(x20d),1]+x20d[!is.na(x20d)])],plot=F,breaks=c(0:12)+0.5)$counts
 h5mm <- aggregate(x5mm[,2],list(100*(x5mm[,1]/100-trunc(x5mm[,1]/100))),mean,na.rm=T)
 h20mm <- aggregate(x20mm[,2],list(100*(x20mm[,1]/100-trunc(x20mm[,1]/100))),mean,na.rm=T)
 hm <- hist(x[!is.na(x[,4]),2],plot=F,breaks=c(0:12)+0.5)$counts
 ### process flood series
 y <- read.table(lst[i,1],header=T)
 if (lst[i,3]=="MJJASO"){selseas <- y[,4]>4&y[,4]<11}else{selseas <- !(y[,4]>4&y[,4]<11)}
 y[!selseas,2] <- NA
 x5dc <- h5m/max(h5m)
 x20dc <- h20m/max(h20m)
 xcmc <- h20mm[,2]/max(h20mm[,2])
 yout <- cbind(1780:2021,NA)
 yout[match(y[,1],1780:2021),2] <- y[,2]
 write.table(yout,str_replace(lst[i,1], ".txt","_flood_season_series.txt"),sep=" ",row.names=F,col.names=F)
 af[,i+1] <- yout[,2]
 yg <- yout
 ymf <- yout[c(1780:2021)>1799&!is.na(yout[,2]),2] ## cut from 1800
 g <- fitdistr(ymf, "gamma")
 yg[!is.na(yout[,2]),2] <- qnorm(pgamma(yout[!is.na(yout[,2]),2],g$estimate[1],g$estimate[2]))
 write.table(yg,str_replace(lst[i,1], ".txt","_flood_season_gamma.txt"),sep=" ",row.names=F,col.names=F)
 afg[,i+1] <- yg[,2]
 }

# write files 
write.table(ap,"all_sites_rx5d.season.txt",sep=" ",row.names=F,col.names=c("yr",lst[,1]))
write.table(apg,"all_sites_rx5d.season.gamma.txt",sep=" ",row.names=F,col.names=c("yr",lst[,1]))
write.table(af,"all_sites_floods.season.txt",sep=" ",row.names=F,col.names=c("yr",lst[,1]))
write.table(afg,"all_sites_floods.season.gamma.txt",sep=" ",row.names=F,col.names=c("yr",lst[,1]))


